"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.apiHandler = exports.HandlingError = void 0;
const config_1 = require("../../config");
const middleware_1 = require("../middleware");
const types_1 = require("./types");
const debug_1 = require("../../debug");
const normalize = (event) => {
    debug_1.log.debug('[API] Normalizing event');
    const clonedEvent = Object.assign(event);
    if (event.body) {
        try {
            debug_1.log.debug('[API] Parsing request body from JSON');
            clonedEvent.body = JSON.parse(clonedEvent.body);
        }
        catch (e) {
            debug_1.log.debug('[API] ERROR: Only JSON Accepted');
            const error = new Error('Only JSON payloads are accepted');
            error.name = 'BadRequestError';
            throw error;
        }
    }
    return clonedEvent;
};
const httpMethodToStatus = (method, statusCode) => {
    debug_1.log.debug('[API] Setting status code for method', method, statusCode || (method === 'POST' ? 201 : 200));
    return statusCode || (method === 'POST' ? 201 : 200);
};
const getHeader = (event, headerName) => {
    for (const header of Object.keys(event.headers)) {
        if (header.toLowerCase() === headerName.toLowerCase()) {
            return event.headers[header];
        }
    }
    return undefined;
};
const singleHeaders = (event, headers) => {
    const finalHeaders = Object.keys(headers)
        .filter((k) => ['boolean', 'string', 'number'].indexOf(typeof headers[k]) > -1)
        .reduce((p, k) => Object.assign(p, { [k]: headers[k] }), {});
    const cors = (0, config_1.getConfig)().api.cors;
    debug_1.log.debug('[API] Reading CORS config', cors);
    if (cors) {
        finalHeaders['Access-Control-Allow-Origin'] = cors.origin ?? getHeader(event, 'Origin');
    }
    return finalHeaders;
};
const multipleHeaders = (event, headers) => {
    const finalHeaders = Object.keys(headers)
        .filter((k) => ['boolean', 'string', 'number'].indexOf(typeof headers[k]) === -1)
        .reduce((p, k) => Object.assign(p, { [k]: headers[k] }), {});
    const cors = (0, config_1.getConfig)().api.cors;
    if (cors) {
        const allowedMethods = ['GET', 'POST', 'PUT', 'PATCH', 'DELETE', 'OPTIONS', 'HEAD'];
        const exposedHeaders = Object.keys(headers).filter((v, i, a) => a.indexOf(v) === i);
        finalHeaders['Access-Control-Allow-Methods'] = cors.methods || allowedMethods;
        finalHeaders['Access-Control-Expose-Headers'] = cors.exposeHeaders || exposedHeaders;
        finalHeaders['Access-Control-Allow-Headers'] = cors.allowHeaders || Object.keys(event.headers);
    }
    return finalHeaders;
};
const format = (event, response, content) => {
    debug_1.log.debug('[API] Formatting response');
    debug_1.log.debug('[API] Setting headers');
    const headers = singleHeaders(event, response.headers);
    const multiValueHeaders = multipleHeaders(event, response.headers);
    const apiGatewayHeaders = {};
    Object.keys(headers).forEach((key) => {
        const value = headers[key];
        if (value) {
            apiGatewayHeaders[key] = value;
        }
    });
    if (content === null || content === undefined || content === '') {
        debug_1.log.debug('[API] No content returning 204');
        return {
            headers: apiGatewayHeaders,
            multiValueHeaders,
            statusCode: event.httpMethod === 'POST' ? 201 : 204,
            body: '',
        };
    }
    return {
        headers: apiGatewayHeaders,
        multiValueHeaders,
        statusCode: httpMethodToStatus(event.httpMethod, response.statusCode),
        body: JSON.stringify(content, (key, value) => {
            return ((0, config_1.getConfig)().api.blacklist ?? []).indexOf(key) > -1 ? undefined : value;
        }),
    };
};
const formatError = (event, response, err) => {
    debug_1.log.debug('[API] Error name', err.name);
    switch (err.name) {
        case 'ValidationError':
            debug_1.log.debug('[API] 422 - Validation error');
            response.statusCode = 422;
            return format(event, response, { data: err.details });
        case 'BadRequestError':
            debug_1.log.debug('[API] 400 - Bad request');
            response.statusCode = 400;
            return format(event, response, err.details ? { data: err.details } : 'Bad Request');
        case 'ForbiddenError':
            response.statusCode = 403;
            return format(event, response, err.details ? { data: err.details } : 'Forbidden');
        default:
            debug_1.log.debug('[API] 500 - Generic error');
            response.statusCode = err.statusCode || 500;
            return format(event, response, err.body || 'Internal Server Error');
    }
};
class HandlingError extends Error {
    statusCode;
    body;
    details;
}
exports.HandlingError = HandlingError;
const apiHandler = (next) => {
    return async (event, context) => {
        debug_1.log.debug('[API] Initializing response');
        const response = new types_1.Response();
        try {
            debug_1.log.debug('[API] Normalizing event', event);
            const normalizedEvent = await normalize(event);
            debug_1.log.debug('[API] Normalized event', normalizedEvent);
            debug_1.log.debug('[API] Calling before middleware');
            await (0, middleware_1.callBeforeMiddleware)('ApiGateway', [normalizedEvent, context]);
            debug_1.log.debug('[API] Run business logic code');
            const result = format(normalizedEvent, response, await next(normalizedEvent, response, context));
            debug_1.log.debug('[API] Formatted result', result);
            debug_1.log.debug('[API] Calling after middleware');
            await (0, middleware_1.callAfterMiddleware)('ApiGateway', [normalizedEvent, result]);
            return result;
        }
        catch (e) {
            const err = e;
            debug_1.log.debug('[API] Error happened !', err);
            const result = formatError(event, response, err);
            debug_1.log.debug('[API] Formatted', result);
            debug_1.log.debug('[API] Calling error middleware');
            await (0, middleware_1.callErrorHandlers)('ApiGateway', [event, err, result]);
            return result;
        }
    };
};
exports.apiHandler = apiHandler;
//# sourceMappingURL=api.js.map